{#- OZI 0.1
# ozi/templates/.gitignore.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-#}
# OZI packaging
sig/
subprojects/ozi/
build*/
/dist/
temp/

# Utility environments
.hypothesis/
.pytest_cache/
.tox/
.ruff_cache/

# coverage files
.coverage
coverage.xml
.coverage.*

# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so