{#- OZI 0.1
# ozi/templates/requirements.in.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-#}
{%- for req in project.dist_requires %}
{{ req }}
{%- endfor %}
